#coding=utf8
import re

from django.db import models
from django.db.models.signals import post_save, post_delete
from django.contrib.auth.models import User

from OpenDirectory.models import Users, Service, Computer
from OpenDirAdmin.settings import OPENDIRADMIN_DOMAIN, POWERDNS_DEFAULT_NS1_LOCATION, POWERDNS_DEFAULT_NS2_LOCATION, POWERDNS_DEFAULT_MX_LOCATION

regexp_ipv6 = re.compile('^((([0-9A-Fa-f]{1,4}:){7}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}:[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){5}:([0-9A-Fa-f]{1,4}:)?[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){4}:([0-9A-Fa-f]{1,4}:){0,2}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){3}:([0-9A-Fa-f]{1,4}:){0,3}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){2}:([0-9A-Fa-f]{1,4}:){0,4}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}((b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|(([0-9A-Fa-f]{1,4}:){0,5}:((b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|(::([0-9A-Fa-f]{1,4}:){0,5}((b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b).){3}(b((25[0-5])|(1d{2})|(2[0-4]d)|(d{1,2}))b))|([0-9A-Fa-f]{1,4}::([0-9A-Fa-f]{1,4}:){0,5}[0-9A-Fa-f]{1,4})|(::([0-9A-Fa-f]{1,4}:){0,6}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,7}:))$')
regexp_ipv4 = re.compile('^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?).(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$')


domain_types = (
        ('master', u'Maître'),
        ('slave', 'Esclave'),
        ('stub', 'Stub'),
        ('forward', 'Proxy'), 
        ('hint', 'Zone cache'),
        )

record_types = (
        ('CNAME', 'Canonical name record (CNAME)'),
        ('A', 'address record (A)'),
        ('AAAA', 'IPv6 address record (AAAA)'),
        ('NS', 'name server record (NS)'),
        ('MX', 'mail exchange record (MX)'),
        ('AFSDB', 'AFS database record (AFSDB)'),
        ('APL', 'Address Prefix List (APL)'),
        ('CERT', 'Certificate record (CERT)'),
        ('DHCID', 'DHCP identifier (DHCID)'),
        ('DLV', 'DNSSEC Lookaside Validation record (DLV)'),
        ('DNAME', 'delegation name (DNAME)'),
        ('DNSKEY', 'DNS Key record (DNSKEY)'),
        ('DS', 'Delegation signer (DS)'),
        ('HIP', 'Host Identity Protocol (HIP)'),
        ('IPSECKEY', 'IPSEC Key (IPSECKEY)'),
        ('KEY', 'key record (KEY)'),
        ('KX', 'Key eXchanger record (KX)'),
        ('LOC', 'Location record (LOC)'),
        ('NAPTR', 'Naming Authority Pointer (NAPTR)'),
        ('NSEC', 'Next-Secure record (NSEC)'),
        ('NSEC3', 'NSEC record version 3 (NSEC3)'),
        ('NSEC3PARAM', 'NSEC3 parameters (NSEC3PARAM)'),
        ('PTR', 'pointer record (PTR)'),
        ('RRSIG', 'DNSSEC signature (RRSIG)'),
        ('RP', 'Responsible person (RP)'),
        ('SIG', 'Signature (SIG)'),
        ('SOA', 'start of [a zone of] authority record (SOA)'),
        ('SPF', 'Sender Policy Framework (SPF)'),
        ('SRV', 'Service locator (SRV)'),
        ('SSHFP', 'SSH Public Key Fingerprint (SSHFP)'),
        ('TA', 'DNSSEC Trust Authorities (TA)'),
        ('TKEY', 'secret key record (TKEY)'),
        ('TSIG', 'Transaction Signature'),
        ('TXT', 'Text record'),
        ('*', 'All cached records'),
        ('AXFR', 'Authoritative Zone Transfer'),
        ('IXFR', 'Incremental Zone Transfer'),
        ('OPT', 'Option'),
        )





class Domains(models.Model):
    name = models.CharField(db_column='name', max_length=255, null=False, blank=False, db_index=True, unique=True, verbose_name=u'Nom de zone')
    master = models.CharField(db_column='master', max_length=128, null=True, blank=True, default=None)
    last_check = models.IntegerField(db_column='last_check', null=True, blank=True, default=None)
    type = models.CharField(db_column='type', max_length=6, null=False, blank=True, choices=domain_types, default=domain_types[0][0])
    notified_serial = models.IntegerField(db_column='notified_serial', null=True, blank=True, default=None)
    account = models.ForeignKey(User, db_column='account', to_field='username', max_length=40, null=True, blank=True, default=None, 
            verbose_name=u'Propriétaire')
    def __unicode__(self):
        return self.name

    class Meta:
        db_table = 'domains'
        ordering = ('name', )
        verbose_name = 'Domaine'
        verbose_name_plural = 'Domaines'

    def create_mandatory_records(self):
        u"""Create required DNS record for this zone"""
# SOA record
        if self.type == 'master' or self.type == 'slave':
            content = '%s %s %d %d %d %d %d' % (POWERDNS_DEFAULT_NS1_LOCATION, 'hostmaster@' + OPENDIRADMIN_DOMAIN, 0, 10800, 3600, 604800, 3600)
            Records(domain=self, name=self.name, type='SOA', content=content, auth=True).save()
# NS1 record
            Records(domain=self, name=self.name, type='NS', content=POWERDNS_DEFAULT_NS1_LOCATION, auth=True).save()
# NS2 record
            Records(domain=self, name=self.name, type='NS', content=POWERDNS_DEFAULT_NS2_LOCATION, auth=True).save()
# MX record
            Records(domain=self, name=self.name, type='MX', content=POWERDNS_DEFAULT_MX_LOCATION, auth=True).save()

class Records(models.Model):
    domain = models.ForeignKey(Domains, db_column='domain_id', blank=True, null=True, db_index=True, default=None)
    name = models.CharField(db_column='name', max_length=255, null=True, blank=True, db_index=True, default=None, verbose_name='Nom')
    type = models.CharField(db_column='type', max_length=6, null=True, blank=True, db_index=True, default=None, choices=record_types)
    content = models.CharField(db_column='content', max_length=255, null=True, blank=True, default=None, verbose_name='Valeur')
    ttl = models.IntegerField(db_column='ttl', null=True, default=86400, blank=True, verbose_name='TTL')
    prio = models.IntegerField(db_column='prio', null=True, blank=True, default=10, verbose_name=u'Priorité')
    change_date = models.IntegerField(db_column='change_date', null=True, default=None, blank=True)
    ordername = models.CharField(db_column='ordername', max_length=255, null=True, blank=True, default=None, db_index=True)
    auth = models.BooleanField(db_column='auth', default=False, verbose_name='Autoritaire')
    def __unicode__(self):
        return "{0} {1}".format(self.name, self.type)
    
    class Meta:
        db_table = 'records'
        ordering = ('name', )
        verbose_name = 'Enregistrement'
        verbose_name_plural = 'Enregistrements'

class Supermasters(models.Model):
    ip = models.CharField(db_column='ip', max_length=25, null=False, primary_key=True)
    nameserver = models.CharField(db_column='nameserver', max_length=255, null=False)
    account = models.ForeignKey(User, db_column='account', to_field='username', max_length=40, null=True, blank=True, default=None, 
            verbose_name=u'Propriétaire')
    
    class Meta:
        db_table = 'supermasters'
        ordering = ('ip', )
        verbose_name = u'Maître'
        verbose_name_plural = u'Maîtres'


class DomainMetadata(models.Model):
    domain = models.ForeignKey(Domains, db_column='domain_id', db_index=True)
    kind = models.CharField(db_column='kind', max_length=16, null=True, blank=True, db_index=True, default=None)
    content = models.TextField(db_column='content', null=True, blank=True, default=None)
    def __unicode__(self):
        return u"{0}.{1}".format(self.domain.name, self.kind)
    
    class Meta:
        db_table = 'domainmetadata'
        ordering = ('domain', 'kind', )
        verbose_name = u'Méta-donnée de domaine'
        verbose_name_plural = u'Méta-données de domaines'


class CryptoKeys(models.Model):
    domain = models.ForeignKey(Domains, db_column='domain_id', db_index=True)
    flags = models.IntegerField(db_column='flags', null=False, blank=False)
    active = models.BooleanField(db_column='active', default=False)
    content = models.TextField(db_column='content', null=True, blank=True, default=None)
    def __unicode__(self):
        return u"{0}.{1}".format(self.domain.name, self.flags)
    
    class Meta:
        db_table = 'cryptokeys'
        ordering = ('domain', 'flags', )
        verbose_name = u'Clef de chiffrement'
        verbose_name_plural = u'Clefs de chiffrement'


class TSigKeys(models.Model):
    name = models.CharField(db_column='name', max_length=255, null=True, blank=True, db_index=True, default=None)
    algorithm = models.CharField(db_column='algorithm', max_length=50, null=True, blank=True, db_index=True, default=None)
    secret = models.CharField(db_column='secret', max_length=255, null=True, blank=True, default=None)



def create_powerdns_domain(sender, instance, created, *args, **kwargs):
    if created:
        instance.create_mandatory_records()

post_save.connect(create_powerdns_domain, Domains)

def reverse_lookup_ptr(sender, instance, created, *args, **kwargs):
    if instance.type not in ('A', 'AAAA'):
        return
    if instance.type == 'A':
        s = instance.content.split('.')
        s.reverse()
        domain_name = '.'.join(s[1:]) + '.in-addr.arpa'
        name = '.'.join(s) + '.in-addr.arpa'
    elif instance.type == 'AAAA':
        s = instance.content.split(':')
        s.reverse()
        domain_name = ':'.join(s[1:]) + '.ip6.arpa'
        name = ':'.join(s) + '.ip6.arpa'
    domain, c = Domains.objects.get_or_create(name=domain_name, master='master')
    record, c = Records.objects.get_or_create(domain=domain, name=name, type='PTR', content=instance.name)

post_save.connect(reverse_lookup_ptr, Records)

def instance_to_dns(dns_names, ips):
    u"""return a list of (name, zone, ip address) """
    result = []
    if dns_names and ips:
        for name in dns_names.split('\n'):
            name = name.strip().lower()
            if name.find('.') < 0: continue
            for ip in ips.split('\n'):
                if not ip.strip() or not name: continue
                result.append((name, ip.strip()))
    return result

def create_powerdns_service(sender, instance, created, *args, **kwargs):
    previous_data = instance_to_dns(instance.o__applednsname, instance.o__iphostnumber)
    new_data = instance_to_dns(instance.applednsname, instance.iphostnumber)
    previous_names = set(x[0] for x in previous_data)
    new_names = set(x[0] for x in new_data)
    for prev_name in previous_names - new_names:
        Records.objects.filter(name=prev_name).delete()
    for (name, ip) in new_data:
        domain_name = name.split('.', 1)[1]
        domain, c = Domains.objects.get_or_create(name=domain_name, type=domain_types[0][0])
        if regexp_ipv4.match(ip):
            type_ = 'A'
        elif regexp_ipv6.match(ip):
            type_ = 'AAAA'
        else:
            type_ = 'CNAME'
        record, c = Records.objects.get_or_create(domain=domain, name=name, content=ip)
        if c:
            record.type = type_
            record.save()
    return True

post_save.connect(create_powerdns_service, Service)

def delete_powerdns_service(sender, instance, *args, **kwargs):
    data = instance_to_dns(instance.applednsname, instance.iphostnumber)
    for name, ip in data:
        Records.objects.filter(name=name).delete()

post_delete.connect(delete_powerdns_service, Service)

def clean_computer_name(name):
    if len(name) == 0:
        return name
    if name[-1] == '$':
        name = name[0:-1]
    if name.find('.') == -1:
        name = u'{0}.{1}.{2}'.format(basename, OPENDIRADMIN_COMPUTER_PREFIX, OPENDIRADMIN_DOMAIN).lower() 
    return name

def create_powerdns_computer(sender, instance, created, *args, **kwargs):
    name = clean_computer_name(instance.name)
    __name = clean_computer_name(instance.o__name)
    if __name != name:
        Records.objects.filter(name=__name).delete()
    domain_name = name.split('.', 1)[1]
    domain, c = Domains.objects.get_or_create(name=domain_name, type=domain_types[0][0])
    for ip in instance.iphostnumber.split('\n'):
        ip = ip.strip().lower()
        if not ip: continue
        type_ = (ip.find(':') >= 0) and 'AAAA' or 'A'
        record, c = Records.objects.get_or_create(domain=domain, name=name, content=ip)
        if c:
            record.type = type_
            record.save()
    return True

post_save.connect(create_powerdns_computer, Computer)

def delete_powerdns_computer(sender, instance, *args, **kwargs):
    name = clean_computer_name(instance.name)
    Records.objects.filter(name=name).delete()

post_delete.connect(delete_powerdns_computer, Computer)
