#coding=utf8

import datetime
import time

import dateutil.parser
import django.db.models
from django.db.models import fields, SubfieldBase
from ldapdb import escape_ldap_filter

class MultilineField(fields.Field):
    __metaclass__ = SubfieldBase

    def from_ldap(self, value, connection):
        return value

    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        return [x.strip().encode(connection.charset) for x in value.split("\n")]

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        value = filter(lambda x: x.strip(), value.split("\n"))
        if lookup_type == 'contains':
            return escape_ldap_filter(value)
        raise TypeError("MultilineField has invalid lookup: %s" % lookup_type)

    def to_python(self, value):
        if not value:
            return ''
        if isinstance(value, list) or isinstance(value, tuple):
            return "\n".join(value)
        return value


class FloatField(fields.FloatField):
    def from_ldap(self, value, connection):
        if len(value) == 0:
            return 0
        else:
            return float(value[0])
        
    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        return [str(value)]

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        if lookup_type in ('exact', 'gte', 'lte'):
            return value
        raise TypeError("FloatField has invalid lookup: %s" % lookup_type)


class DateTimeField(fields.DateTimeField):
    __metaclass__ = SubfieldBase

    def from_ldap(self, value, connection):
        if not value:
            return None
        if not isinstance(value, list) and not isinstance(value, tuple):
            value = (value, )
        return [dateutil.parser.parse('{0}-{1}-{2} {3}:{4}:{5}'.format(x[0:4], x[4:6], x[6:8], x[8:10], x[10:12], x[12:])) for x in value]

    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        if not isinstance(value, list) and not isinstance(value, tuple):
            value = (value, )
        return [x.strftime('%Y%m%d%H%M%S%f%z') for x in value]

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        if lookup_type in ('exact', 'gte', 'lte'):
            return value
        raise TypeError("DateTimeField has invalid lookup: %s" % lookup_type)

    def to_python(self, value):
        if not value:
            return []
        return value


class BooleanField(fields.BooleanField):
    __metaclass__ = SubfieldBase

    def from_ldap(self, value, connection):
        if not value: return False
        if isinstance(value, list) or isinstance(value, tuple):
            return value[0].lower() != 'false'
        return value and value.lower() != 'false'

    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        return [value and 'TRUE' or 'FALSE']

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        if lookup_type in ('exact'):
            return value
        raise TypeError("BooleanField has invalid lookup: %s" % lookup_type)


class OctetStringField(fields.TextField):
    def from_ldap(self, value, connection):
        if len(value) == 0:
            return ''
        else:
            return value[0]

    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        return [value]

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        raise TypeError("OctetStringField has invalid lookup: %s" % lookup_type)


class ForeignKey(django.db.models.ForeignKey):
    def from_ldap(self, value, connection):
        if len(value) == 0:
            return None
        else:
            return value[0]
        
    def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
        "Returns field's value prepared for database lookup."
        return [self.get_prep_lookup(lookup_type, value)]

    def get_db_prep_save(self, value, connection):
        if value:
            return [str(value.__getattribute__(self.rel.field_name))]
        return []

    def get_prep_lookup(self, lookup_type, value):
        "Perform preliminary non-db specific lookup checks and conversions"
        if lookup_type in ('exact', 'gte', 'lte'):
            return value
        raise TypeError("FloatField has invalid lookup: %s" % lookup_type)

    def to_python(self, value):
        if not value:
            return []
        return value
